import {
	AnyComponentSelectMenuInteraction,
	ComponentInteraction,
	ComponentTypes,
	InteractionTypes,
	MessageFlags,
	ModalSubmitInteraction,
	TextInputStyles
} from "oceanic.js";
import { client } from "..";
import { canUser } from "../utils/utils";
import { selfappReq } from "../utils/selfappReq";
import { Response } from "~/utils/types";
import { db } from "~/utils/database";
import { ComponentMessage } from "components-jsx/ComponentMessage";
import { Container } from "components-jsx/Container";
import { Divider } from "components-jsx/Divider";
import { TextDisplay } from "components-jsx/TextDisplay";
import { ApplicationContent } from "~/components/ApplicationContent";
import { User } from "~/components/User";
import { Constants } from "~/Constants";
import { TextInput } from "components-jsx/TextInput";
import { ActionRow } from "components-jsx/ActionRow";
import { rejectReasons } from "~/utils/rejectReasons";

export async function rejectUser(
	interaction: ComponentInteraction | ModalSubmitInteraction,
	reason: string = ""
) {
	await interaction.defer();
	const shouldBan = interaction.data.customID.includes("ban");
	const application = await db.get(
		"SELECT * FROM applications WHERE id=?",
		interaction.data.customID.split("-")[
			interaction.data.customID.split("-").length - 2
		]
	);

	const user = await client.rest.users.get(application.user_id);
	await interaction.createFollowup({
		content: `${
			shouldBan
				? "<:BAN:1375806319621046313>"
				: "<:blobcatred:1375806202470203513>"
		} **Rejected** \`@${user.tag}\`'s join request${
			shouldBan ? " and banned them" : ""
		}!`
	});

	const response: Response = JSON.parse(application.responses);

	const msg = await client.rest.channels.editMessage(
		application.channel_id,
		application.message_id,
		<ComponentMessage>
			<Container accentColor={Constants.COLORS.BAD}>
				<TextDisplay>## Join request rejected</TextDisplay>
				<Divider />
				<User user={user} />
				<Divider />
				<ApplicationContent response={response} includeDetails={true} />
				<Divider />
				<TextDisplay>
					### Reason
					<br />
					{reason}
				</TextDisplay>
			</Container>
		</ComponentMessage>
	);

	await db.run(
		`UPDATE applications SET status=-1, reject_reason=?, actioned_by=?, message_id=? WHERE id=?`,
		reason,
		interaction.member!.id,
		msg.id,
		interaction.data.customID.split("-")[
			interaction.data.customID.split("-").length - 1
		]
	);

	await selfappReq(
		`/guilds/${interaction.guildID}/requests/id/${application.id}`,
		"PATCH",
		{
			action: "REJECTED",
			rejection_reason: reason
		}
	);
	shouldBan &&
		(await interaction.guild?.createBan(user.id, {
			reason
		}));

	await client.rest.channels.edit(application.thread_id, {
		locked: true,
		reason: `${user.tag}'s application has been rejected`,
		appliedTags: [Constants.TAGS.REJECTED]
	});
}

client.on("interactionCreate", async (interaction) => {
	if (interaction.type === InteractionTypes.MODAL_SUBMIT) {
		await rejectUser(
			interaction,
			`${interaction.member?.tag}: ${
				interaction.data.components.getTextInput("reason") ||
				"No reason provided"
			}`
		);
	}
	if (interaction.type === InteractionTypes.MESSAGE_COMPONENT) {
		if (interaction.data.componentType === ComponentTypes.BUTTON) {
			if (interaction.data.customID.split("-")[0] === "reject") {
				if (!canUser(interaction.member!, "review"))
					return await interaction.createFollowup({
						flags: MessageFlags.EPHEMERAL,
						content: "💢 nop"
					});

				await interaction.createModal({
					title: `Reject ${
						interaction.data.customID.includes("ban")
							? "and ban "
							: ""
					}user`,
					components: [
						<ActionRow>
							<TextInput
								customID="reason"
								label="Reason"
								style={TextInputStyles.PARAGRAPH}
								maxLength={126}
								required={false}
							/>
						</ActionRow>
					],
					customID: interaction.data.customID
				});
			}
		}
		if (interaction.data.componentType === ComponentTypes.STRING_SELECT) {
			await rejectUser(
				interaction,
				`${interaction.member!.tag}: ${
					rejectReasons[interaction.data.values.getStrings()[0]]
						.reason
				}`
			);
		}
	}
});
